@@ -2,7 +2,7 @@
 /* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2008, 2009 Gustavo Noronha Silva
- *  Copyright © 2009, 2010 Igalia S.L.
+ *  Copyright © 2009, 2010, 2014 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "ephy-file-monitor.h"
 #include "ephy-form-auth-data.h"
 #include "ephy-history-service.h"
+#include "ephy-lib-type-builtins.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
@@ -65,7 +66,7 @@
 #define EPHY_PAGE_TEMPLATE_ERROR         "/org/gnome/epiphany/page-templates/error.html"
 
 struct _EphyWebViewPrivate {
-  EphyWebViewSecurityLevel security_level;
+  EphySecurityLevel security_level;
   EphyWebViewDocumentType document_type;
   EphyWebViewNavigationFlags nav_flags;
 
@@ -107,6 +108,9 @@ struct _EphyWebViewPrivate {
   GTlsCertificate *certificate;
   GTlsCertificateFlags tls_errors;
 
+  gboolean loading_tls_error_page;
+  char *tls_error_page_host;
+
   /* Web Extension */
   EphyWebExtensionProxy *web_extension;
 };
@@ -695,6 +699,25 @@ form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
   gtk_widget_show (info_bar);
 }
 
+static void
+allow_tls_certificate_cb (EphyWebExtensionProxy *shell,
+                          guint64 page_id,
+                          EphyWebView *web_view)
+{
+  EphyWebViewPrivate *priv = web_view->priv;
+
+  if (webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (web_view)) != page_id)
+    return;
+
+  g_return_if_fail (G_IS_TLS_CERTIFICATE (priv->certificate));
+  g_return_if_fail (priv->tls_error_page_host != NULL);
+
+  webkit_web_context_allow_tls_certificate_for_host (webkit_web_context_get_default (),
+                                                     priv->certificate,
+                                                     priv->tls_error_page_host);
+  ephy_web_view_load_url (web_view, ephy_web_view_get_address (web_view));
+}
+
 static void
 page_created_cb (EphyEmbedShell *shell,
                  guint64 page_id,
@@ -712,6 +735,10 @@ page_created_cb (EphyEmbedShell *shell,
   g_signal_connect (priv->web_extension, "form-auth-data-save-requested",
                     G_CALLBACK (form_auth_data_save_requested),
                     web_view);
+
+  g_signal_connect (priv->web_extension, "allow-tls-certificate",
+                    G_CALLBACK (allow_tls_certificate_cb),
+                    web_view);
 }
 
 static void
@@ -757,6 +784,7 @@ ephy_web_view_finalize (GObject *object)
   g_free (priv->typed_address);
   g_free (priv->link_message);
   g_free (priv->loading_message);
+  g_free (priv->tls_error_page_host);
 
   G_OBJECT_CLASS (ephy_web_view_parent_class)->finalize (object);
 }
@@ -947,15 +975,15 @@ ephy_web_view_class_init (EphyWebViewClass *klass)
 /**
  * EphyWebView:security-level:
  *
- * One of #EphyWebViewSecurityLevel, determining view's current security level.
+ * One of #EphySecurityLevel, determining view's current security level.
  **/
   g_object_class_install_property (gobject_class,
                                    PROP_SECURITY,
                                    g_param_spec_enum ("security-level",
                                                       "Security Level",
                                                       "The view's security level",
-                                                      EPHY_TYPE_WEB_VIEW_SECURITY_LEVEL,
-                                                      EPHY_WEB_VIEW_STATE_IS_UNKNOWN,
+                                                      EPHY_TYPE_SECURITY_LEVEL,
+                                                      EPHY_SECURITY_LEVEL_NO_SECURITY,
                                                       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 /**
@@ -1573,7 +1601,7 @@ load_changed_cb (WebKitWebView *web_view,
     break;
   case WEBKIT_LOAD_COMMITTED: {
     const char* uri;
-    EphyWebViewSecurityLevel security_level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
+    EphySecurityLevel security_level = EPHY_SECURITY_LEVEL_NO_SECURITY;
 
     priv->ever_committed = TRUE;
 
@@ -1582,14 +1610,20 @@ load_changed_cb (WebKitWebView *web_view,
     ephy_web_view_location_changed (view, uri);
 
     /* Security status. */
-    g_clear_object (&priv->certificate);
-    if (webkit_web_view_get_tls_info (web_view, &priv->certificate, &priv->tls_errors)) {
-      g_object_ref (priv->certificate);
-      security_level = priv->tls_errors == 0 ?
-        EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH : EPHY_WEB_VIEW_STATE_IS_BROKEN;
-    }
+    if (priv->loading_tls_error_page) {
+      priv->loading_tls_error_page = FALSE;
+    } else {
+      g_clear_object (&priv->certificate);
+      g_clear_pointer (&priv->tls_error_page_host, g_free);
 
-    ephy_web_view_set_security_level (EPHY_WEB_VIEW (web_view), security_level);
+      if (webkit_web_view_get_tls_info (web_view, &priv->certificate, &priv->tls_errors)) {
+        g_object_ref (priv->certificate);
+        security_level = priv->tls_errors == 0 ?
+          EPHY_SECURITY_LEVEL_STRONG_SECURITY : EPHY_SECURITY_LEVEL_BROKEN_SECURITY;
+      }
+
+      ephy_web_view_set_security_level (EPHY_WEB_VIEW (web_view), security_level);
+    }
 
     /* History. */
     if (!ephy_web_view_is_history_frozen (view)) {
@@ -1691,6 +1725,104 @@ get_style_sheet (void)
   return sheet;
 }
 
+static char *
+detailed_message_from_tls_errors (GTlsCertificateFlags tls_errors)
+{
+  GPtrArray *errors = g_ptr_array_new ();
+  char *retval;
+
+  if (tls_errors & G_TLS_CERTIFICATE_BAD_IDENTITY) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site presented identification that belongs to a different web site."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_EXPIRED) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification is too old to trust. Check the date on your computer’s calendar."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_UNKNOWN_CA) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification was not issued by a trusted organization."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_GENERIC_ERROR) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification could not be processed. It may be corrupted."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_REVOKED) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification has been revoked by the trusted organization that issued it."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_INSECURE) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification cannot be trusted because it uses very weak encryption."));
+  }
+
+  if (tls_errors & G_TLS_CERTIFICATE_NOT_ACTIVATED) {
+    /* Possible error message when a site presents a bad certificate. */
+    g_ptr_array_add (errors, _("This web site’s identification is only valid for future dates. Check the date on your computer’s calendar."));
+  }
+
+  if (errors->len == 1) {
+    retval = g_strdup (g_ptr_array_index (errors, 0));
+  } else if (errors->len > 1) {
+    GString *message = g_string_new ("<ul>");
+    guint i;
+
+    for (i = 0; i < errors->len; i++) {
+      g_string_append_printf (message, "<li>%s</li>", (char *)g_ptr_array_index (errors, i));
+    }
+
+    g_string_append (message, "</ul>");
+    retval = g_string_free (message, FALSE);
+  } else {
+    g_assert_not_reached ();
+  }
+
+  g_ptr_array_free (errors, TRUE);
+
+  return retval;
+}
+
+static char *
+get_tls_error_page_message (EphyWebView *view, const char *hostname)
+{
+  EphyWebViewPrivate *priv = view->priv;
+  char *msg;
+  char *bold_hostname;
+  char *details;
+  char *warning;
+
+  bold_hostname = g_strconcat ("<strong>", hostname, "</strong>", NULL);
+  details = detailed_message_from_tls_errors (priv->tls_errors);
+  /* Message when a site's TLS certificate is invalid. %s is the site's hostname. */
+  warning = g_strdup_printf (_("This might not be the real %s."), bold_hostname);
+
+  msg = g_strdup_printf ("<p>%s</p><p>%s</p><p>%s</p><p>%s <strong>%s</strong></p>",
+                         warning,
+                         /* Message when a site's TLS certificate is invalid. */
+                         _("When you try to connect securely, web sites present "
+                           "identification to prove that your connection has not been "
+                           "maliciously intercepted. There is something wrong with "
+                           "this site’s identification:"),
+                         details,
+                         /* Message when a site's TLS certificate is invalid. */
+                         _("A third party may have hijacked your connection. You should "
+                           "continue only if you know there is a good reason why this "
+                           "site does not use trusted identification."),
+                         /* Good advice from Firefox; displays when a site's TLS certificate is invalid. */
+                         _("Legitimate banks, stores, and other public sites will "
+                           "not ask you to do this."));
+  g_free (bold_hostname);
+  g_free (details);
+  g_free (warning);
+
+  return msg;
+}
+
 /**
  * ephy_web_view_load_error_page:
  * @view: an #EphyWebView
@@ -1716,6 +1848,7 @@ ephy_web_view_load_error_page (EphyWebView *view,
   char *msg;
   char *button_label;
   char *stylesheet;
+  char *load_anyway_js = NULL;
   const char *custom_class;
   GBytes *html_file;
 
@@ -1768,6 +1901,17 @@ ephy_web_view_load_error_page (EphyWebView *view,
       button_label = g_strdup (_("Reload Anyway"));
       custom_class = "process-crash";
       break;
+    case EPHY_WEB_VIEW_ERROR_INVALID_TLS_CERTIFICATE:
+      /* Page title when a site cannot be loaded. %s is the site's hostname. */
+      page_title = g_strdup_printf (_("Problem loading “%s”"), hostname);
+      /* Title of error page when a website's TLS certificate is invalid. */
+      msg_title = g_strdup (_("Look out!"));
+      msg = get_tls_error_page_message (view, hostname);
+      /* Button on error page when a website's TLS certificate is invalid. */
+      button_label = g_strdup (_("Load Anyway"));
+      custom_class = "tls-error";
+      load_anyway_js = g_strdup ("EpiphanyTLSCertificateErrorPage.allowException();");
+      break;
     default:
       return;
       break;
@@ -1777,13 +1921,17 @@ ephy_web_view_load_error_page (EphyWebView *view,
   _ephy_web_view_update_icon (view);
 
   stylesheet = get_style_sheet ();
+
+  if (load_anyway_js == NULL)
+    load_anyway_js = g_strdup_printf ("window.location = '%s';", uri);
+
   g_string_printf (html,
                    g_bytes_get_data (html_file, NULL),
                    lang, lang,
                    ((gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) ? "rtl" : "ltr"),
                    page_title,
                    stylesheet,
-                   uri,
+                   load_anyway_js,
                    custom_class,
                    msg_title, msg, button_label);
 
@@ -1791,6 +1939,7 @@ ephy_web_view_load_error_page (EphyWebView *view,
   g_free (stylesheet);
   g_free (lang);
   g_free (page_title);
+  g_free (load_anyway_js);
   g_free (msg_title);
   g_free (msg);
   g_free (button_label);
@@ -1866,6 +2015,43 @@ load_failed_cb (WebKitWebView *web_view,
   return FALSE;
 }
 
+static gboolean
+load_failed_with_tls_error_cb (WebKitWebView *web_view,
+                               GTlsCertificate *certificate,
+                               GTlsCertificateFlags errors,
+                               gchar *host,
+                               gpointer user_data)
+{
+  EphyWebView *view = EPHY_WEB_VIEW (web_view);
+  EphyWebViewPrivate *priv = view->priv;
+
+  g_clear_object (&priv->certificate);
+  g_clear_pointer (&priv->tls_error_page_host, g_free);
+
+  priv->loading_tls_error_page = TRUE;
+  priv->certificate = g_object_ref (certificate);
+  priv->tls_errors = errors;
+  priv->tls_error_page_host = g_strdup (host);
+  ephy_web_view_set_security_level (EPHY_WEB_VIEW (web_view), EPHY_SECURITY_LEVEL_BROKEN_SECURITY);
+  ephy_web_view_load_error_page (EPHY_WEB_VIEW (web_view),
+                                 webkit_web_view_get_uri (web_view),
+                                 EPHY_WEB_VIEW_ERROR_INVALID_TLS_CERTIFICATE, NULL);
+
+  return TRUE;
+}
+
+static void
+mixed_content_detected_cb (WebKitWebView *web_view,
+                           WebKitInsecureContentEvent event,
+                           gpointer user_data)
+{
+  EphyWebView *view = EPHY_WEB_VIEW (web_view);
+  EphyWebViewPrivate *priv = view->priv;
+
+  if (priv->security_level != EPHY_SECURITY_LEVEL_BROKEN_SECURITY)
+    ephy_web_view_set_security_level (view, EPHY_SECURITY_LEVEL_MIXED_CONTENT);
+}
+
 static void
 close_web_view_cb (WebKitWebView *web_view,
                    gpointer user_data)
@@ -1915,7 +2101,7 @@ ephy_web_view_init (EphyWebView *web_view)
   priv->is_blank = TRUE;
   priv->ever_committed = FALSE;
   priv->document_type = EPHY_WEB_VIEW_DOCUMENT_HTML;
-  priv->security_level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
+  priv->security_level = EPHY_SECURITY_LEVEL_NO_SECURITY;
 
   priv->file_monitor = ephy_file_monitor_new (web_view);
 
@@ -1945,6 +2131,14 @@ ephy_web_view_init (EphyWebView *web_view)
                     G_CALLBACK (load_failed_cb),
                     NULL);
 
+  g_signal_connect (web_view, "load-failed-with-tls-errors",
+                    G_CALLBACK (load_failed_with_tls_error_cb),
+                    NULL);
+
+  g_signal_connect (web_view, "insecure-content-detected",
+                    G_CALLBACK (mixed_content_detected_cb),
+                    NULL);
+
   g_signal_connect (web_view, "notify::zoom-level",
                     G_CALLBACK (zoom_changed_cb),
                     NULL);
@@ -1990,7 +2184,8 @@ GtkWidget *
 ephy_web_view_new (void)
 {
   return g_object_new (EPHY_TYPE_WEB_VIEW,
-                       "group", ephy_embed_prefs_get_web_view_group (),
+                       "user-content-manager", ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ()),
+                       "settings", ephy_embed_prefs_get_settings (),
                        NULL);
 }
 
@@ -1999,7 +2194,7 @@ ephy_web_view_new_with_related_view (WebKitWebView *related_view)
 {
   return g_object_new (EPHY_TYPE_WEB_VIEW,
                        "related-view", related_view,
-                       "group", ephy_embed_prefs_get_web_view_group (),
+                       "settings", ephy_embed_prefs_get_settings (),
                        NULL);
 }
 
@@ -2295,13 +2490,13 @@ ephy_web_view_set_link_message (EphyWebView *view,
 /**
  * ephy_web_view_set_security_level:
  * @view: an #EphyWebView
- * @level: the new #EphyWebViewSecurityLevel for @view
+ * @level: the new #EphySecurityLevel for @view
  *
  * Sets @view's security-level property to @level.
  **/
 void
 ephy_web_view_set_security_level (EphyWebView *view,
-                                  EphyWebViewSecurityLevel level)
+                                  EphySecurityLevel level)
 {
   EphyWebViewPrivate *priv = view->priv;
 
@@ -2581,13 +2776,13 @@ ephy_web_view_get_web_app_title_finish (EphyWebView *view,
  * @certificate: (out) (transfer none): return value of TLS certificate
  * @errors: (out): return value of TLS errors
  *
- * Fetches the #EphyWebViewSecurityLevel and a #GTlsCertificate associated
+ * Fetches the #EphySecurityLevel and a #GTlsCertificate associated
  * with @view and a #GTlsCertificateFlags showing what problems, if any,
  * have been found with that certificate.
  **/
 void
 ephy_web_view_get_security_level (EphyWebView *view,
-                                  EphyWebViewSecurityLevel *level,
+                                  EphySecurityLevel *level,
                                   GTlsCertificate **certificate,
                                   GTlsCertificateFlags *errors)
 {
