@@ -50,6 +50,11 @@ struct _UriTesterPrivate
 
   GString *blockcss;
   GString *blockcssprivate;
+
+  GRegex *regex_third_party;
+  GRegex *regex_pattern;
+  GRegex *regex_subdocument;
+  GRegex *regex_frame_add;
 };
 
 enum
@@ -66,7 +71,7 @@ G_DEFINE_TYPE (UriTester, uri_tester, G_TYPE_OBJECT)
 static GString *
 uri_tester_fixup_regexp (const char *prefix, char *src);
 
-static gboolean
+static void
 uri_tester_parse_file_at_uri (UriTester *tester, const char *fileuri);
 
 static char *
@@ -303,8 +308,7 @@ uri_tester_check_rule (UriTester  *tester,
     return FALSE;
 
   opts = g_hash_table_lookup (tester->priv->optslist, patt);
-  if (opts && g_regex_match_simple (",third-party", opts,
-                                    G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY))
+  if (opts && g_regex_match (tester->priv->regex_third_party, opts, 0, NULL))
     {
       if (page_uri && g_regex_match_full (regex, page_uri, -1, 0, 0, NULL, NULL))
         return FALSE;
@@ -459,7 +463,7 @@ uri_tester_fixup_regexp (const char *prefix, char *src)
   return str;
 }
 
-static gboolean
+static void
 uri_tester_compile_regexp (UriTester *tester,
                            GString   *gpatt,
                            char      *opts)
@@ -470,7 +474,7 @@ uri_tester_compile_regexp (UriTester *tester,
   int len;
 
   if (!gpatt)
-    return FALSE;
+    return;
 
   patt = gpatt->str;
   len = gpatt->len;
@@ -482,10 +486,10 @@ uri_tester_compile_regexp (UriTester *tester,
     {
       g_warning ("%s: %s", G_STRFUNC, error->message);
       g_error_free (error);
-      return TRUE;
+      return;
     }
 
-  if (!g_regex_match_simple ("^/.*[\\^\\$\\*].*/$", patt, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY))
+  if (!g_regex_match (tester->priv->regex_pattern, patt, 0, NULL))
     {
       int signature_count = 0;
       int pos = 0;
@@ -493,7 +497,7 @@ uri_tester_compile_regexp (UriTester *tester,
 
       for (pos = len - SIGNATURE_SIZE; pos >= 0; pos--) {
         sig = g_strndup (patt + pos, SIGNATURE_SIZE);
-        if (!g_regex_match_simple ("[\\*]", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
+        if (!strchr (sig, '*') &&
             !g_hash_table_lookup (tester->priv->keys, sig))
           {
             LOG ("sig: %s %s", sig, patt);
@@ -503,7 +507,7 @@ uri_tester_compile_regexp (UriTester *tester,
           }
         else
           {
-            if (g_regex_match_simple ("^\\*", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
+            if (sig[0] == '*' &&
                 !g_hash_table_lookup (tester->priv->pattern, patt))
               {
                 LOG ("patt2: %s %s", sig, patt);
@@ -516,12 +520,7 @@ uri_tester_compile_regexp (UriTester *tester,
       g_regex_unref (regex);
 
       if (signature_count > 1 && g_hash_table_lookup (tester->priv->pattern, patt))
-        {
-          g_hash_table_steal (tester->priv->pattern, patt);
-          return TRUE;
-        }
-
-      return FALSE;
+        g_hash_table_steal (tester->priv->pattern, patt);
     }
   else
     {
@@ -529,11 +528,10 @@ uri_tester_compile_regexp (UriTester *tester,
       /* Pattern is a regexp chars */
       g_hash_table_insert (tester->priv->pattern, g_strdup (patt), regex);
       g_hash_table_insert (tester->priv->optslist, g_strdup (patt), g_strdup (opts));
-      return FALSE;
     }
 }
 
-static char*
+static void
 uri_tester_add_url_pattern (UriTester *tester,
                             char      *prefix,
                             char      *type,
@@ -543,13 +541,12 @@ uri_tester_add_url_pattern (UriTester *tester,
     char *patt;
     GString *format_patt;
     char *opts;
-    gboolean should_free;
 
     data = g_strsplit (line, "$", -1);
     if (!data || !data[0])
     {
         g_strfreev (data);
-        return NULL;
+        return;
     }
 
     if (data[1] && data[2])
@@ -568,21 +565,20 @@ uri_tester_add_url_pattern (UriTester *tester,
         opts = type;
     }
 
-    if (g_regex_match_simple ("subdocument", opts,
-                              G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY))
+    if (g_regex_match (tester->priv->regex_subdocument, opts, 0, NULL))
     {
         if (data[1] && data[2])
             g_free (patt);
         if (data[1])
             g_free (opts);
         g_strfreev (data);
-        return NULL;
+        return;
     }
 
     format_patt = uri_tester_fixup_regexp (prefix, patt);
 
     LOG ("got: %s opts %s", format_patt->str, opts);
-    should_free = uri_tester_compile_regexp (tester, format_patt, opts);
+    uri_tester_compile_regexp (tester, format_patt, opts);
 
     if (data[1] && data[2])
         g_free (patt);
@@ -590,7 +586,7 @@ uri_tester_add_url_pattern (UriTester *tester,
         g_free (opts);
     g_strfreev (data);
 
-    return g_string_free (format_patt, should_free);
+    g_string_free (format_patt, TRUE);
 }
 
 static inline void
@@ -602,8 +598,7 @@ uri_tester_frame_add (UriTester *tester, char *line)
   (void)*line++;
   if (strchr (line, '\'')
       || (strchr (line, ':')
-          && !g_regex_match_simple (".*\\[.*:.*\\].*", line,
-                                    G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY)))
+          && !g_regex_match (tester->priv->regex_frame_add, line, 0, NULL)))
     {
       return;
     }
@@ -622,8 +617,7 @@ uri_tester_frame_add_private (UriTester  *tester,
   if (!(data[1] && *data[1])
       ||  strchr (data[1], '\'')
       || (strchr (data[1], ':')
-          && !g_regex_match_simple (".*\\[.*:.*\\].*", data[1],
-                                    G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY)))
+          && !g_regex_match (tester->priv->regex_frame_add, data[1], 0, NULL)))
     {
       g_strfreev (data);
       return;
@@ -650,87 +644,129 @@ uri_tester_frame_add_private (UriTester  *tester,
   g_strfreev (data);
 }
 
-static char*
+static void
 uri_tester_parse_line (UriTester *tester, char *line)
 {
   if (!line)
-    return NULL;
+    return;
+
   g_strchomp (line);
   /* Ignore comments and new lines */
   if (line[0] == '!')
-    return NULL;
+    return;
   /* FIXME: No support for whitelisting */
   if (line[0] == '@' && line[1] == '@')
-    return NULL;
+    return;
   /* FIXME: No support for [include] and [exclude] tags */
   if (line[0] == '[')
-    return NULL;
+    return;
   /* FIXME: No support for domain= */
   if (strstr (line, "domain="))
-    return NULL;
+    return;
 
   /* Skip garbage */
   if (line[0] == ' ' || !line[0])
-    return NULL;
+    return;
 
   /* Got CSS block hider */
   if (line[0] == '#' && line[1] == '#' )
     {
       uri_tester_frame_add (tester, line);
-      return NULL;
+      return;
     }
   /* Got CSS block hider. Workaround */
   if (line[0] == '#')
-    return NULL;
+    return;
 
   /* Got per domain CSS hider rule */
   if (strstr (line, "##"))
     {
       uri_tester_frame_add_private (tester, line, "##");
-      return NULL;
+      return;
     }
 
   /* Got per domain CSS hider rule. Workaround */
   if (strchr (line, '#'))
     {
       uri_tester_frame_add_private (tester, line, "#");
-      return NULL;
+      return;
     }
   /* Got URL blocker rule */
   if (line[0] == '|' && line[1] == '|' )
     {
       (void)*line++;
       (void)*line++;
-      return uri_tester_add_url_pattern (tester, "", "fulluri", line);
+      uri_tester_add_url_pattern (tester, "", "fulluri", line);
+      return;
     }
   if (line[0] == '|')
     {
       (void)*line++;
-      return uri_tester_add_url_pattern (tester, "^", "fulluri", line);
+      uri_tester_add_url_pattern (tester, "^", "fulluri", line);
+      return;
     }
-  return uri_tester_add_url_pattern (tester, "", "uri", line);
+  uri_tester_add_url_pattern (tester, "", "uri", line);
 }
 
-static gboolean
+static void
+file_parse_cb (GDataInputStream *stream, GAsyncResult *result, UriTester *tester)
+{
+  char *line;
+  GError *error = NULL;
+
+  line = g_data_input_stream_read_line_finish (stream, result, NULL, &error);
+  if (!line) {
+    if (error) {
+      LOG ("Error parsing file: %s\n", error->message);
+      g_error_free (error);
+    }
+
+    return;
+  }
+
+  uri_tester_parse_line (tester, line);
+  g_free (line);
+
+  g_data_input_stream_read_line_async (stream, G_PRIORITY_DEFAULT_IDLE, NULL,
+                                       (GAsyncReadyCallback)file_parse_cb, tester);
+}
+
+static void
+file_read_cb (GFile *file, GAsyncResult *result, UriTester *tester)
+{
+  GFileInputStream *stream;
+  GDataInputStream *data_stream;
+  GError *error = NULL;
+
+  stream = g_file_read_finish (file, result, &error);
+  if (!stream) {
+    char *path;
+
+    path = g_file_get_path (file);
+    LOG ("Error opening file %s for parsing: %s\n", path, error->message);
+    g_free (path);
+    g_error_free (error);
+
+    return;
+  }
+
+  data_stream = g_data_input_stream_new (G_INPUT_STREAM (stream));
+  g_object_unref (stream);
+
+  g_data_input_stream_read_line_async (data_stream, G_PRIORITY_DEFAULT_IDLE, NULL,
+                                       (GAsyncReadyCallback)file_parse_cb, tester);
+  g_object_unref (data_stream);
+}
+
+static void
 uri_tester_parse_file_at_uri (UriTester *tester, const char *fileuri)
 {
-  FILE *file;
-  char line[2000];
-  char *path = NULL;
-  gboolean result = FALSE;
+  GFile *file;
+  GFileInputStream *stream;
 
-  path = g_filename_from_uri (fileuri, NULL, NULL);
-  if ((file = g_fopen (path, "r")))
-    {
-      while (fgets (line, 2000, file))
-        g_free (uri_tester_parse_line (tester, line));
-      fclose (file);
-
-      result = TRUE;
-    }
-  g_free (path);
-
-  return result;
+  file = g_file_new_for_uri (fileuri);
+  g_file_read_async (file, G_PRIORITY_DEFAULT_IDLE, NULL, (GAsyncReadyCallback)file_read_cb, tester);
+  g_object_unref (file);
 }
 
 static void
@@ -759,6 +795,23 @@ uri_tester_init (UriTester *tester)
 
   priv->blockcss = g_string_new ("z-non-exist");
   priv->blockcssprivate = g_string_new ("");
+
+  priv->regex_third_party = g_regex_new (",third-party",
+                                         G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+                                         G_REGEX_MATCH_NOTEMPTY,
+                                         NULL);
+  priv->regex_pattern = g_regex_new ("^/.*[\\^\\$\\*].*/$",
+                                     G_REGEX_UNGREEDY | G_REGEX_OPTIMIZE,
+                                     G_REGEX_MATCH_NOTEMPTY,
+                                     NULL);
+  priv->regex_subdocument = g_regex_new ("subdocument",
+                                         G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+                                         G_REGEX_MATCH_NOTEMPTY,
+                                         NULL);
+  priv->regex_frame_add = g_regex_new (".*\\[.*:.*\\].*",
+                                       G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+                                       G_REGEX_MATCH_NOTEMPTY,
+                                       NULL);
 }
 
 static void
@@ -813,6 +866,11 @@ uri_tester_finalize (GObject *object)
   g_string_free (priv->blockcss, TRUE);
   g_string_free (priv->blockcssprivate, TRUE);
 
+  g_regex_unref (priv->regex_third_party);
+  g_regex_unref (priv->regex_pattern);
+  g_regex_unref (priv->regex_subdocument);
+  g_regex_unref (priv->regex_frame_add);
+
   G_OBJECT_CLASS (uri_tester_parent_class)->finalize (object);
 }
 
