@@ -44,6 +44,14 @@ enum {
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
+enum
+{
+  LOCK_CLICKED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 typedef struct
 {
   EphyWindow    *window;
@@ -148,39 +156,10 @@ static void
 ephy_title_box_add_address_bar (EphyTitleBox *title_box)
 {
   EphyTitleBoxPrivate *priv = ephy_title_box_get_instance_private (title_box);
-  GtkActionGroup      *action_group;
-  GtkAction           *action;
-  GtkWidget           *box;
-  GtkWidget           *reload;
-  GtkSizeGroup        *size;
-
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_widget_show (box);
-  gtk_style_context_add_class (gtk_widget_get_style_context (box), "location-entry");
-  gtk_style_context_add_class (gtk_widget_get_style_context (box), GTK_STYLE_CLASS_LINKED);
-  gtk_stack_add_named (GTK_STACK (title_box), box, "address-bar");
 
   priv->entry = ephy_location_entry_new ();
   gtk_widget_show (priv->entry);
-  gtk_box_pack_start (GTK_BOX (box), priv->entry, TRUE, TRUE, 0);
-
-  /* Reload/Stop */
-  reload = gtk_button_new ();
-  gtk_button_set_focus_on_click (GTK_BUTTON (reload), FALSE);
-  gtk_widget_show (reload);
-  /* FIXME: apparently we need an image inside the button for the action
-  * icon to appear. */
-  gtk_button_set_image (GTK_BUTTON (reload), gtk_image_new ());
-  gtk_widget_set_valign (reload, GTK_ALIGN_CENTER);
-  action_group = ephy_window_get_toolbar_action_group (priv->window);
-  action = gtk_action_group_get_action (action_group, "ViewCombinedStopReload");
-  gtk_activatable_set_related_action (GTK_ACTIVATABLE (reload), action);
-  gtk_container_add (GTK_CONTAINER (box), reload);
-
-  size = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
-  gtk_size_group_add_widget (size, priv->entry);
-  gtk_size_group_add_widget (size, reload);
-  g_object_unref (size);
+  gtk_stack_add_named (GTK_STACK (title_box), priv->entry, "address-bar");
 }
 
 static void
@@ -208,7 +187,6 @@ ephy_title_box_add_title_bar (EphyTitleBox *title_box)
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   context = gtk_widget_get_style_context (hbox);
   gtk_style_context_add_class (context, "subtitle");
-  gtk_style_context_add_class (context, "dim-label");
   gtk_widget_set_halign (hbox, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (hbox, GTK_ALIGN_BASELINE);
   gtk_widget_show (hbox);
@@ -356,17 +334,24 @@ ephy_title_box_button_press_event (GtkWidget      *widget,
 {
   EphyTitleBox        *title_box = EPHY_TITLE_BOX (widget);
   EphyTitleBoxPrivate *priv = ephy_title_box_get_instance_private (title_box);
+  GtkAllocation        lock_allocation;
 
   if (priv->mode != EPHY_TITLE_BOX_MODE_TITLE
-      || event->button != GDK_BUTTON_PRIMARY
-      || priv->location_disabled)
+      || event->button != GDK_BUTTON_PRIMARY)
     return GDK_EVENT_PROPAGATE;
 
   LOG ("button-press-event title-box %p event %p", title_box, event);
 
-  if (event->type == GDK_BUTTON_PRESS) {
+  gtk_widget_get_allocation (priv->lock_image, &lock_allocation);
+
+  if (event->x >= lock_allocation.x &&
+      event->x < lock_allocation.x + lock_allocation.width &&
+      event->y >= lock_allocation.y &&
+      event->y < lock_allocation.y + lock_allocation.height) {
+    g_signal_emit (title_box, signals[LOCK_CLICKED], 0, (GdkRectangle *)&lock_allocation);
+  } else if (!priv->location_disabled && event->type == GDK_BUTTON_PRESS) {
     priv->button_down = TRUE;
-  } else {
+  } else if (!priv->location_disabled) {
     priv->button_down = FALSE;
     ephy_title_box_cancel_switch_to_entry_after_double_click_time (title_box);
   }
@@ -444,6 +429,23 @@ ephy_title_box_class_init (EphyTitleBoxClass *klass)
   g_object_class_install_properties (object_class,
                                      N_PROPERTIES,
                                      object_properties);
+
+  /**
+   * EphyTitleBox::lock-clicked:
+   * @title_box: the object on which the signal is emitted
+   * @lock_position: the position of the lock icon
+   *
+   * Emitted when the user clicks the security icon inside the
+   * #EphyTitleBox.
+   */
+  signals[LOCK_CLICKED] = g_signal_new ("lock-clicked",
+                                        EPHY_TYPE_TITLE_BOX,
+                                        G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+                                        0, NULL, NULL,
+                                        g_cclosure_marshal_generic,
+                                        G_TYPE_NONE,
+                                        1,
+                                        GDK_TYPE_RECTANGLE | G_SIGNAL_TYPE_STATIC_SCOPE);
 }
 
 static void
@@ -481,14 +483,20 @@ ephy_title_box_transform_uri_to_label (GBinding     *binding,
                                        GValue       *to_value,
                                        gpointer      user_data)
 {
-  const gchar *label;
-  gboolean     rtl;
-  gchar       *uri;
-
-  rtl = gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL;
+  const gchar       *label;
+  gchar             *uri;
+  EphyEmbedShellMode mode;
 
   label = g_value_get_string (from_value);
-  uri = g_strconcat (rtl ? "▾ " : label, rtl ? label : " ▾", NULL);
+  mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
+
+  if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    uri = g_strdup (label);
+  } else {
+    gboolean rtl;
+    rtl = gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL;
+    uri = g_strconcat (rtl ? "▾ " : label, rtl ? label : " ▾", NULL);
+  }
   g_value_take_string (to_value, uri);
 
   return TRUE;
@@ -552,15 +560,15 @@ ephy_title_box_set_web_view (EphyTitleBox  *title_box,
     g_clear_object (&priv->title_binding);
     g_clear_object (&priv->uri_binding);
 
-    g_object_unref (priv->web_view);
+    g_object_remove_weak_pointer (G_OBJECT (priv->web_view), (gpointer *)&priv->web_view);
   }
 
-  priv->web_view = NULL;
+  priv->web_view = web_view;
 
   if (web_view == NULL)
     return;
 
-  priv->web_view = g_object_ref (web_view);
+  g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&priv->web_view);
 
   title = webkit_web_view_get_title (web_view);
 
@@ -650,20 +658,15 @@ ephy_title_box_set_mode (EphyTitleBox    *title_box,
 }
 
 /**
- * ephy_title_box_set_show_lock:
+ * ephy_title_box_set_security_level:
  * @title_box: an #EphyTitleBox
- * @show_lock: if @title_box should show a lock icon indicating the security level
- * of the page
- *
- * If @show_lock is TRUE, the title will show an icon reflecting the
- * security level of the page, by default it's shown only in secure and
- * insecure pages (insecure meaning secure pages with something broken in such
- * security).
+ * @mode: an #EphySecurityLevel
  *
+ * Sets the lock icon to be displayed by the title box and location entry
  **/
 void
-ephy_title_box_set_show_lock (EphyTitleBox *title_box,
-                              gboolean      show_lock)
+ephy_title_box_set_security_level (EphyTitleBox         *title_box,
+                                   EphySecurityLevel     security_level)
 {
   EphyTitleBoxPrivate *priv;
 
@@ -671,34 +674,12 @@ ephy_title_box_set_show_lock (EphyTitleBox *title_box,
 
   priv = ephy_title_box_get_instance_private (title_box);
 
-  gtk_widget_set_visible (priv->lock_image, show_lock);
+  g_object_set (priv->lock_image,
+                "icon-name", ephy_security_level_to_icon_name (security_level),
+                NULL);
 
-  ephy_location_entry_set_show_lock (EPHY_LOCATION_ENTRY (priv->entry), show_lock);
-}
-
-/**
- * ephy_title_box_set_lock_state:
- * @title_box: an #EphyTitleBox
- * @mode: an #EphyLocationLockState
- *
- * Set the lock icon to be displayed, to actually show the icon see
- * ephy_title_box_set_show_lock.
- **/
-void
-ephy_title_box_set_lock_state (EphyTitleBox         *title_box,
-                               EphyLocationLockState state)
-{
-  EphyTitleBoxPrivate *priv;
-
-  g_return_if_fail (EPHY_IS_TITLE_BOX (title_box));
-
-  priv = ephy_title_box_get_instance_private (title_box);
-
-  g_object_set (priv->lock_image, "icon-name", state == EPHY_LOCATION_LOCK_STATE_SECURE ?
-                                               "channel-secure-symbolic" : "channel-insecure-symbolic",
-                                  NULL);
-
-  ephy_location_entry_set_lock_state (EPHY_LOCATION_ENTRY (priv->entry), state);
+  gtk_widget_set_visible (priv->lock_image, security_level != EPHY_SECURITY_LEVEL_NO_SECURITY);
+  ephy_location_entry_set_security_level (EPHY_LOCATION_ENTRY (priv->entry), security_level);
 }
 
 /**
